//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 実数寸法ヘッダ
 * @author Junpee
 */

#ifndef DIMENSION_F_H_
#define DIMENSION_F_H_

namespace Lamp{

class DimensionI;

//------------------------------------------------------------------------------
/**
 * 実数寸法
 *
 * このクラスは継承しないで下さい。
 */
class DimensionF{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// 幅
			float width;
			/// 高さ
			float height;
		};

		/// 配列
		float array[2];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ寸法
	static const DimensionF zero;

	/// 単位寸法
	static const DimensionF unit;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	DimensionF(){}

	/**
	 * コンストラクタ
	 * @param sourceWidth 幅の初期値
	 * @param sourceHeight 高さの初期値
	 */
	DimensionF(float sourceWidth, float sourceHeight) :
		width(sourceWidth), height(sourceHeight){}

	/**
	 * コンストラクタ
	 * @param sourceArray 初期値配列
	 */
	explicit DimensionF(float sourceArray[2]) :
		width(sourceArray[0]), height(sourceArray[1]){}

	/**
	 * コンストラクタ
	 * @param source 設定する寸法
	 */
	explicit DimensionF(const DimensionI& source);

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceWidth 幅の設定値
	 * @param sourceHeight 高さの設定値
	 */
	inline void set(float sourceWidth, float sourceHeight){
		width = sourceWidth;
		height = sourceHeight;
	}

	/**
	 * 値の設定
	 * @param sourceArray 設定値配列
	 */
	inline void set(float sourceArray[2]){
		width = sourceArray[0];
		height = sourceArray[1];
	}

	/**
	 * 値の設定
	 * @param source 設定する寸法
	 */
	void set(const DimensionI& source);

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addDimension 加算する寸法
	 * @return 加算された寸法
	 */
	inline DimensionF operator +(const DimensionF& addDimension) const{
		return DimensionF(width + addDimension.width,
			height + addDimension.height);
	}

	/**
	 * 減算
	 * @param subDimension 減算する寸法
	 * @return 減算された寸法
	 */
	inline DimensionF operator -(const DimensionF& subDimension) const{
		return DimensionF(width - subDimension.width,
			height - subDimension.height);
	}

	/**
	 * 乗算
	 * @param mulDimension 乗算する寸法
	 * @return 乗算された寸法
	 */
	inline DimensionF operator *(const DimensionF& mulDimension) const{
		return DimensionF(width * mulDimension.width,
			height * mulDimension.height);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された寸法
	 */
	inline DimensionF operator *(float mulValue) const{
		return DimensionF(width * mulValue, height * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulDimension 乗算される寸法
	 * @return 乗算された寸法
	 */
	inline friend DimensionF operator *(
		float mulValue, const DimensionF& mulDimension){
		return DimensionF(mulDimension.width * mulValue,
			mulDimension.height * mulValue);
	}

	/**
	 * +演算子
	 * @return ベクトルのコピー
	 */
	inline DimensionF operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転したベクトル
	 */
	inline DimensionF operator -() const{ return DimensionF(-width, -height); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addDimension 加算する寸法
	 * @return 加算された寸法
	 */
	inline DimensionF& operator +=(const DimensionF& addDimension){
		width += addDimension.width;
		height += addDimension.height;
		return (*this);
	}

	/**
	 * 代入減算
	 * @param subDimension 減算する寸法
	 * @return 減算された寸法
	 */
	inline DimensionF& operator -=(const DimensionF& subDimension){
		width -= subDimension.width;
		height -= subDimension.height;
		return (*this);
	}

	/**
	 * 代入乗算
	 * @param mulDimension 乗算する寸法
	 * @return 乗算された寸法
	 */
	inline DimensionF& operator *=(const DimensionF& mulDimension){
		width *= mulDimension.width;
		height *= mulDimension.height;
		return (*this);
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された寸法
	 */
	inline DimensionF& operator *=(float mulValue){
		width *= mulValue;
		height *= mulValue;
		return (*this);
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較する寸法
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const DimensionF& target) const{
		return ((width == target.width) && (height == target.height));
	}

	/**
	 * 同じ値かどうか
	 * @param target 比較する寸法
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(const DimensionF& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(width - target.width) <= epsilon) &&
			(Math::abs(height - target.height) <= epsilon));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する寸法
	 * @return 同じ値でなければtrueを返す
	 */
	inline bool operator !=(const DimensionF& target) const{
		return ((width != target.width) || (height != target.height));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する寸法
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const DimensionF& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(width - target.width) > epsilon) ||
			(Math::abs(height - target.height) > epsilon));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 寸法の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f )", width, height);
		return returnString;
	}

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of DIMENSION_F_H_
//------------------------------------------------------------------------------
